/*************************************************************************/
/* libsurl - library for generating shortened URLs                       */
/* Copyright (C) 2009  Scott Lawrence <bytbox@gmail.com> et al.          */
/*                                                                       */
/* This program is free software: you can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation, either version 3 of the License, or (at */
/* your option) any later version.                                       */
/*                                                                       */
/* This program is distributed in the hope that it will be useful, but   */
/* WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     */
/* General Public License for more details.                              */
/*                                                                       */
/* You should have received a copy of the GNU General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/*************************************************************************/

/* curl_simple.c - simple curl wrapper library, custom-tailored for
   libsurl engines */

#include "libsurl-fe.h"

#include <curl/curl.h>
#include <stdlib.h>
#include <string.h>

size_t easy_http_request0(void *ptr,size_t size,size_t nmemb,void *stream) {
    SURL *surl=(SURL *)stream;
    strncat(surl->buffer,ptr,nmemb);
    return nmemb;
}

char *easy_http_request(SURL *surl,char *url,char *postdata) {
    if(surl->buffer)
        free(surl->buffer);
    surl->buffer_usage=0;
    surl->buffer_size=SURL_CURL_MAX_BUFFER_SIZE;
    surl->buffer=(char *)malloc(surl->buffer_size);
    surl->buffer[0]=0;

    CURL *curl=curl_easy_init();
    curl_easy_setopt(curl,CURLOPT_USERAGENT, "libsurl (https://launchpad.net/libsurl)");
    curl_easy_setopt(curl,CURLOPT_TIMEOUT_MS,3000);
    curl_easy_setopt(curl,CURLOPT_WRITEDATA,surl);
    curl_easy_setopt(curl,CURLOPT_WRITEFUNCTION,easy_http_request0);
    curl_easy_setopt(curl,CURLOPT_URL,url);
    if(postdata)
        curl_easy_setopt(curl,CURLOPT_POSTFIELDS,postdata);
    curl_easy_setopt(curl,CURLOPT_FOLLOWLOCATION,1);
    curl_easy_setopt(curl,CURLOPT_MAXREDIRS,10);
    curl_easy_perform(curl);
    curl_easy_cleanup(curl);

    return surl->buffer;
}
