/*************************************************************************/
/* libsurl - library for generating shortened URLs                       */
/* Copyright (C) 2009  Scott Lawrence <bytbox@gmail.com> et al.          */
/*                                                                       */
/* This program is free software: you can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation, either version 3 of the License, or (at */
/* your option) any later version.                                       */
/*                                                                       */
/* This program is distributed in the hope that it will be useful, but   */
/* WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     */
/* General Public License for more details.                              */
/*                                                                       */
/* You should have received a copy of the GNU General Public License     */
/* along with this program.  If not, see <http://www.gnu.org/licenses/>. */
/*************************************************************************/

/* engine.c - engine management */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "engine.h"
#include "curl_simple.h"

/**********************************/
/* Utilities for template parsing */
/**********************************/

/* create a request from the template and the SURL object */
char *surl_request_template_subst(SURL *surl,char *tmpl) {
    if(tmpl==0) return "";
    //allocations
    char *r=(char *)malloc(2000); // the request string (also return value)
    int tmpllen=strlen(tmpl); // template length
    int i,pos=0;
    for(i=0;i<2000;i++) // early segfault prevention
        r[i]=0;

    for(i=0;i<tmpllen;i++) {
        if(tmpl[i]!='\\') // regular stuff
            r[pos++]=tmpl[i];
        else { // escape sequence
            i++;
            switch(tmpl[i]) {
            case 'u': //url
                strcat(r,surl_getopt(surl,SURLOPT_URL));
                break;
            case 'k': //api key
                strcat(r,surl_getopt(surl,SURLOPT_API_KEY));
                break;
            case 'n': //username
                strcat(r,surl_getopt(surl,SURLOPT_USERNAME));
                break;
            case 'p': //password
                strcat(r,surl_getopt(surl,SURLOPT_PASSWORD));
                break;
            case 't': //title
                strcat(r,surl_getopt(surl,SURLOPT_TITLE));
                break;
            }
            pos=strlen(r);
        }
    }
    return r;
}

/* returns 1 if c is a valid URL character, 0 otherwise */
int isURLChar(char c) {
    return 
        (c>='A' && c<='Z') || //lower case
        (c>='a' && c<='z') || //upper case
        (c>='0' && c<='9') || //numberic
        c==':' || c=='/' || c=='_' || c=='-' || c=='.' || c==','; //misc
}

/* Extract the url from the data using the given response template */
char *surl_apply_response_template(char *tmpl,char *data) {
    int datalen=strlen(data),tmpllen=strlen(tmpl);
    int i,loc=-1;

    //locate the URL
    for(i=0;i<datalen;i++) 
        if(!strncmp(tmpl,data+i,tmpllen)) {
            loc=i+tmpllen;
            break;
        }
    if(loc==-1) return 0;

    //copy the URL
    char *r=(char *)malloc(200);
    int a=0;
    for(i=loc;isURLChar(data[i]) && i<datalen;i++) {
        r[a++]=data[i];
    }
    r[a]=0;
    return r;
}


/**********************/
/* Engine definitions */
/**********************/

void surl_engine_default_shortener(SURL *surl,void *data) {
    surl_engine_def_t *def=(surl_engine_def_t *)data;
    surl->error=0;
    
    //generate the request string
    char *request_string=surl_request_template_subst(surl,def->request_template);
    char *post_string=0;
    if(def->post_template) {
        post_string=surl_request_template_subst(surl,def->post_template);
        if(surl_debug(surl)) //debug block
            fprintf(stderr,"Post string is: %s\n",post_string);
    }
    if(surl_debug(surl)) //debug block
        fprintf(stderr,"Request string is: %s\n",request_string);

    //get and process the result
    char *result=easy_http_request(surl,request_string,post_string);
    if(surl_debug(surl)) //debug block
        fprintf(stderr,"Result\n=======\n%s\n======\n",result);
    char *url=surl_apply_response_template(def->response_template,result);
    surl_setopt(surl,SURLOPT_SHORT_URL,url); //return
}

void surl_engine_bit_ly(SURL *surl,void *data) {
    char *api_key=surl_getopt(surl,SURLOPT_API_KEY);
    char *username=surl_getopt(surl,SURLOPT_USERNAME);
    char *title=surl_getopt(surl,SURLOPT_TITLE);
    char *url=surl_getopt(surl,SURLOPT_URL);
    surl->error=0;
    if(!username) { // we needa username for this site
        surl->error=SURLERR_USERNAME;
        return;
    }
    if(!api_key) { // we need a api key for this site
        surl->error=SURLERR_API_KEY;
        return;
    }
    if(!url) { // well doh!
        surl->error=SURLERR_URL;
        return;
    }
    char *request_string=(char *)malloc(strlen(url)+strlen(api_key)+500);
    if(!title) title="";
    sprintf(request_string,
            "http://api.bit.ly/shorten?version=2.0.1&login=%s&apiKey=%s&longUrl=%s&format=json",
            username,api_key,url);
    char *result=easy_http_request(surl,request_string,0);
    //puts(result);
    int i,len=strlen(result);
    for(i=0;i<len;i++)
        if(!strncmp("shortUrl\": ",result+i,10)) {
            break;
        }
    i+=12;
    char *c;
    char *s=(char *)malloc(80);
    for(c=result+i;c[0]!='\"' && c[0]>0;c++)
        s[c-result-i]=c[0];
    surl_setopt(surl,SURLOPT_SHORT_URL,s);
}

void surl_engine_turl_ca(SURL *surl,void *data) {
    char *url=surl_getopt(surl,SURLOPT_URL);
    surl->error=0;
    if(!url) { // we need a url
        surl->error=SURLERR_URL;
        return;
    }
    char *request_string=(char *)malloc(strlen(url)+500);
    sprintf(request_string,
            "http://turl.ca/api.php?url=%s",
            url);
    char *result=easy_http_request(surl,request_string,0);
    int i,len=strlen(result);
    for(i=0;result[i]!=':' && i<len;i++)
        if(result[i]=='R') {
            surl->error=-1;
            return;
        }
    
    i++;
    char *c;
    char *s=(char *)malloc(40+len);
    for(c=result+i;c[0]!=0;c++)
        s[c-result-i]=c[0];
    s[c-result-i]=0;
    char *sc=(char *)malloc(40+len);
    sprintf(sc,"http://turl.ca/%s",s);
    surl_setopt(surl,SURLOPT_SHORT_URL,sc);
}

void surl_engine_decenturl(SURL *surl,void *data) {
    char *url=surl_getopt(surl,SURLOPT_URL);
    surl->error=0;
    if(!url) {
        surl->error=SURLERR_URL;
        return;
    }
    char *request_string=(char *)malloc(strlen(url)+500);
    sprintf(request_string,
            "http://decenturl.com/api-get?u=%s",url);
    char *result=easy_http_request(surl,request_string,0);
    if(surl_debug(surl)) //debug block
        fprintf(stderr,"Result\n=======\n%s\n======\n",result);

    int i,len=strlen(result);
    
    //back up until we hit a comma
    for(i=len-1;result[i]!=',' && i>0;i--);
    if(i==0) return;
    //forward until we find a quote
    for(;result[i]!='\"';i++);
    i++;
    char *s=(char *)malloc(len);
    int loc=0;
    for(;result[i]!='\"';i++)
        s[loc++]=result[i];
    s[loc]=0;
    surl_setopt(surl,SURLOPT_SHORT_URL,s);
}


/*********************/
/* Engine management */
/*********************/

surl_engine_t *surl_engine_init() {
    surl_engine_t *engine=(surl_engine_t *)malloc(sizeof(surl_engine_t));
    engine->data=0;
    engine->shortener=surl_engine_default_shortener;
    return engine;
}

surl_engine_t *surl_engine_raw_create(surl_shortener_t shortener) {
    surl_engine_t *engine=surl_engine_init();
    engine->shortener=shortener;
    return engine;
}

surl_engine_t *surl_engine_create(surl_engine_def_t *def) {
    surl_engine_t *engine=surl_engine_init();
    engine->data=def;
    return engine;
}

surl_engine_def_t *surl_engine_def_create(char *a,char *b,char *c) {
    surl_engine_def_t *def=(surl_engine_def_t *)malloc(sizeof(surl_engine_def_t));
    def->request_template=a;
    def->post_template=b;
    def->response_template=c;
    int i;
    for(i=0;i<SURLOPT_COUNT;i++)
        def->required[i]=0;
    def->required[SURLOPT_URL]=1;
    return def;
}

void surl_register_minimal_get_engine(SURL *surl,char *service,char *template) {
    surl_register_engine(surl,service,
                         surl_engine_create(surl_engine_def_create(template,0,"")));
}

void surl_register_minimal_post_engine(SURL *surl,char *service,char *url,char *data) {
    surl_register_engine(surl,service,
                         surl_engine_create(surl_engine_def_create(url,data,"")));
}

void surl_engine_cleanup(surl_engine_t *engine) {
    if(engine->data)
        free(engine->data);
    free(engine);
}


/******************************/
/* The actual engine creation */
/******************************/

#define SRMGE(x,y) surl_register_minimal_get_engine(surl,x,y)
#define SRMPE(x,y,z) surl_register_minimal_post_engine(surl,x,y,z)

void surl_register_builtin_engines(SURL *surl) {
    surl_register_engine(surl,BIT_LY,surl_engine_raw_create(surl_engine_bit_ly));
    surl_register_engine(surl,IS_GD, // does not support title
                         surl_engine_create(surl_engine_def_create("http://is.gd/api.php?longurl=\\u",0,"")));
    surl_register_engine(surl,TINYURL, // no API documentation available
                         surl_engine_create(surl_engine_def_create("http://tinyurl.com/api-create.php?url=\\u",0,"")));
    //surl_register_engine(surl,BURNURL,
    //                      surl_engine_create(surl_engine_def_create("http://burnurl.com/?url=\\u&output=plain",0,"")));
    SRMGE(CLI_GS,
          "http://cli.gs/api/v1/cligs/create?appid=libsurl&title=\\t&url=\\u");
    surl_register_engine(surl,DECENTURL, // requires manual foo
                         surl_engine_raw_create(surl_engine_decenturl));
    surl_register_engine(surl,DIGG, // bad api
                         surl_engine_create(surl_engine_def_create("http://services.digg.com/url/short/create?appkey=http://libsurl.sourceforge.net/&url=\\u",0,"short_url=\"")));
    SRMGE(KL_AM, //does not support title
          "http://kl.am/api/shorten/?url=\\u&format=text");
    SRMGE(LIIP_TO,"http://liip.to/api/txt/?url=\\u");
    surl_register_engine(surl,NOTLONG,
                         surl_engine_create(surl_engine_def_create("http://notlong.com/","nickname=\\t&url=\\u","\n<a href=\"")));
    SRMGE(METAMARK,"http://metamark.net/api/rest/simple?long_url=\\u");
    SRMPE(SN_IM,"http://sn.im/site/getsnip","sniplink=\\u&snipformat=simple&snipuser=\\n&snipapi=\\k");
    SRMPE(SNIPR,"http://snipr.com/site/getsnip","sniplink=\\u&snipformat=simple&snipuser=\\n&snipapi=\\k");
    SRMPE(SNIPURL,"http://snipurl.com/site/getsnip","sniplink=\\u&snipformat=simple&snipuser=\\n&snipapi=\\k");
    SRMPE(SNURL,"http://snurl.com/site/getsnip","sniplink=\\u&snipformat=simple&snipuser=\\n&snipapi=\\k");
    SRMGE(TR_IM,"http://api.tr.im/v1/trim_simple?url=\\u");
    surl_register_engine(surl,TURL_CA,surl_engine_raw_create(surl_engine_turl_ca));
    SRMGE(UR_LY,"http://ur.ly/new.txt?href=\\u");
    surl_register_engine(surl,URLOF,
                         surl_engine_create(surl_engine_def_create("http://www.urlof.com/?url=\\u",0,"\n  <a href=\"")));
    SRMGE(ZZ_GD,"http://zz.gd/api-create.php?url=\\u");
    SRMPE(A_GD,"http://a.gd/?module=ShortURL&file=Add&mode=API","url=\\u");
    SRMPE(A_NF,"http://a.nf/?module=ShortURL&file=Add&mode=API","url=\\u");
    SRMGE(CHILP_IT,"http://chilp.it/api.php?url=\\u");
    SRMGE(KURL_IT,"http://kurl.it/api/create/?url=\\u");
    SRMGE(PIURL,"http://www.piurl.com/api.php?url=\\u");
    SRMGE(RI_MS,"http://ri.ms/api-create.php?url=\\u");
    SRMGE(SRNK_NET,"http://srnk.net/create.php?url=\\u");
    surl_register_engine(surl,TINY_CC,
                         surl_engine_create(surl_engine_def_create("http://tiny.cc/","fullurl=\\u","copy(\"")));
    SRMPE(TWURL_NL,"http://tweetburner.com/links","link[url]=\\u");
    SRMGE(YIT_ME,"http://yit.me/api.php?url=\\u");
    SRMGE(ZI_MA,"http://zi.ma/?module=ShortURL&file=Add&mode=API&url=\\u");
}
